@@ -28,6 +28,7 @@
 #ifndef BWT_BNTSEQ_H
 #define BWT_BNTSEQ_H
 
+#include <assert.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <zlib.h>
@@ -75,6 +76,8 @@ extern "C" {
 	int bns_pos2rid(const bntseq_t *bns, int64_t pos_f);
 	int bns_cnt_ambi(const bntseq_t *bns, int64_t pos_f, int len, int *ref_id);
 	uint8_t *bns_get_seq(int64_t l_pac, const uint8_t *pac, int64_t beg, int64_t end, int64_t *len);
+	uint8_t *bns_fetch_seq(const bntseq_t *bns, const uint8_t *pac, int64_t *beg, int64_t mid, int64_t *end, int *rid);
+	int bns_intv2rid(const bntseq_t *bns, int64_t rb, int64_t re);
 
 #ifdef __cplusplus
 }
